#@IgnoreInspection BashAddShebang
# String functionality (Koalephant Shell Script Library)
# Version: PACKAGE_VERSION
# Copyright: 2014-2019, Koalephant Co., Ltd
# Author: Stephen Reay <stephen@koalephant.com>, Koalephant Packaging Team <packages@koalephant.com>


# Parse a string into an integer
# Accepts decimal or hex values, any precision (right of the decimal point) is lost
#
# Input:
# $1 - the string to parse
# $2 - the default value to return if the string is not an integer
#
# Output:
# an integer or the default value if specified
#
# Return:
# 0 if either the first or second argument can be parsed to int, 1 if not
k_int_parse() {
	# shellcheck disable=SC2039
	local string int default
	k_requires_args k_int_parse $#

	string="${1}"
	default="${2:-}"

	if ! k_int_parse_error "${string}" 2>/dev/null && [ -n "${default}" ] && ! k_int_parse_error "${default}" 2>/dev/null; then
		k_log_err 'Invalid default integer value: "%s"' "${default}"
		return 1
	fi
}

# Parse a string into an integer or error
# Accepts decimal or hex values, any precision (right of the decimal point) is lost
#
# Input:
# $1 - the string to parse
#
# Output:
# an integer
# Return:
# 0 if the string can be parsed to an integer 1 if not
k_int_parse_error() {
	# shellcheck disable=SC2039
	local string int
	k_requires_args k_int_parse_error $#

	string="$(k_string_trim "$(k_string_remove_end ".[0-9]*" "$1")")" # Cannot use [[:digit:]] in posh or mksh

	if [ -n "${string}" ]; then
		if int="$(printf -- '%d' "${string}" 2>/dev/null)"; then
			printf -- '%d' "${int}"
			return 0
		fi
	fi

	k_log_err 'Cannot parse "%s" as integer' "${1}"
	return 1
}
